/*
 * 22/04/2002
 *
 * CanvasJeu
 * Auteur: Vouriot Thierry
 * Email: thierry.vouriot@free.fr, yeri@fr.st
 * Web: http://www.yeri.fr.st
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
import java.awt.*;
import java.awt.event.*;

/** Class permettant l'affichage de la grille du jeu de la vie */
public final class CanvasJeu extends Canvas {
    
    private boolean[][] tab;
    private int width=0;
    private int height;
    private int offX;
    private int offY;
    private Image bufferImg = null;
    private Graphics bufferGraph = null;
    
    /** Constructeur par dfaut */
    public CanvasJeu() { }
    
    /** Change le tableau de boolens qui contient les cellules vivantes et mortes 
     *  @param t nouveau tableau
     */
    synchronized public void setTab(boolean[][] t) {
        tab = t;
    }
    
    /** Initialise quelques variables sur les graphismes
     *  @param g contexte graphique
     */
    public void initVariables(Graphics g) {
        Dimension dim = this.getSize();
        width = dim.width;
        height = dim.height;
        bufferImg = this.createImage(width, height);
        bufferGraph = bufferImg.getGraphics();  
    }
    
    /* Fonction qui ractualise l'affichage (ncessaire pour l'implantation
     * de la double buffrisation)
     * @param g contexte graphique
     */
    public void update(Graphics g) {
        offX = (width - (JdVie.zoom*(JdVie.TAILLE)))/2 + JdVie.offsetX;
        offY = (height - (JdVie.zoom*(JdVie.TAILLE)))/2 + JdVie.offsetY;
        paint(bufferGraph);
        g.drawImage(bufferImg,0,0,null);
    }
    
    /** Fonction appele pour dessiner le jeu de la vie
     *  @param g contexte graphique
     */
    public void paint(Graphics g) {
        int k, j;
        g.setColor(Color.black);
        g.fillRect(0, 0, width, height);
        
	//Dessin de la grille
        g.setColor(Color.red);
        for (k=0 ; k<=JdVie.TAILLE; k++) {
            g.drawLine(k*JdVie.zoom+offX, offY, k*JdVie.zoom+offX, JdVie.TAILLE*JdVie.zoom+offY);
            g.drawLine(offX, k*JdVie.zoom+offY, JdVie.TAILLE*JdVie.zoom+offX, k*JdVie.zoom+offY);
        }
        
	//Dessin des cellules vivantes
        g.setColor(Color.white);
        synchronized(this) {
            for (k=0; k<JdVie.TAILLE; k++)
                for (j=0; j<JdVie.TAILLE; j++)
                    if (tab[k][j])
                        g.fillRect(k*JdVie.zoom+offX, j*JdVie.zoom+offY, JdVie.zoom, JdVie.zoom);  
        }
    }
}
