/*
 * 22/04/2002
 *
 * Generateur
 * Auteur: Vouriot Thierry
 * Email: thierry.vouriot@free.fr, yeri@fr.st
 * Web: http://www.yeri.fr.st
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
/** Class permettant de calculer les diffrentes gnrations du jeu de la vie */
public final class Generateur extends Thread {
   
    private boolean[][] tab1, tab2;
    private CanvasJeu jeu;
    private boolean suspendRequest;
    private Integer dummy = new Integer(0);
    
    /** Constructeur
     *  @param t1 tableau du jeu de la vie 1
     *  @param t2 tableau du jeu de la vie 2
     *  @param j zone d'affichage du jeu
     */
    public Generateur(boolean[][] t1, boolean[][] t2, CanvasJeu j) {
        tab1 = t1;
        tab2 = t2;
        jeu = j;
    }
    
    /** Fonction qui calcule le nombre de cellules voisines d'une cellule
     *  dont les coordonnes sont donnes en paramtre
     *  @param x coordonne X
     *  @param y coordonne Y
     */
    private int nbVoisins(int x, int y) {
        int nb = 0;     
        if (y>0 && tab1[x][y-1]) nb++;
        if (y<(JdVie.TAILLE-1) && tab1[x][y+1]) nb++;
        if (x>0) {
            if (y>0 && tab1[x-1][y-1]) nb++;
            if (tab1[x-1][y]) nb++;
            if (y<(JdVie.TAILLE-1) && tab1[x-1][y+1]) nb++;
        }
        if (x<(JdVie.TAILLE-1)) {
            if (y>0 && tab1[x+1][y-1]) nb++;
            if (tab1[x+1][y]) nb++;
            if (y<(JdVie.TAILLE-1) && tab1[x+1][y+1]) nb++;
        }
        return nb;
    }
    
    /** Fonction qui calcule une nouvelle gnration */
    private void generation() {
        int x, y, n;
        for (x=0; x<JdVie.TAILLE; x++)
            for (y=0; y<JdVie.TAILLE; y++) {
                n = nbVoisins(x, y);
                if (n<2 || n>3) tab2[x][y] = false;
                else if (n==3) tab2[x][y] = true;
                else tab2[x][y] = tab1[x][y];
            }
    }
    
    /** Fonction run() qui lance le jeu de la vie */
    public void run() {
        boolean[][] tmp;
        while(true) {
            try {
                checkSuspended();
                sleep(JdVie.vitesse);
            }
            catch(InterruptedException ie) { return; }
            synchronized(this) {
                generation();
                tmp = tab2;
                tab2 = tab1;
                tab1 = tmp;
                jeu.setTab(tab1);
            }
            jeu.repaint();
            JdVie.lblGeneration.setText("Gen n"+ (++JdVie.nbGen));   
        }
    }
    
    /** Suspend l'excution du jeu */
    public void suspendRequest() {
        suspendRequest = true;
    }
    
    /** Reprend l'excution du jeu */
    public void resumeRequest() {
        suspendRequest = false;
        synchronized(dummy) { dummy.notify(); }
    }
    
    /** Vrifie si le jeu est en pause */
    private void checkSuspended() throws InterruptedException {
        synchronized(dummy) {
            while (suspendRequest)
                dummy.wait();
        }
    }
}
