/*
 * 22/04/2002
 *
 * JdVie
 * Auteur: Vouriot Thierry
 * Email: thierry.vouriot@free.fr, yeri@fr.st
 * Web: http://www.yeri.fr.st
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.util.StringTokenizer;
import java.io.*;

/** Applet simulant le jeu de la vie */
public class JdVie extends Applet implements MouseListener, ActionListener, AdjustmentListener {
    private boolean[][] tab1, tab2;
    private Button cmdPlay;
    private Button cmdNew;
    private Button cmdCenter;
    
    private CanvasJeu jeu;
    private Generateur gen;
    
    private Scrollbar scrollZoom;
    private Scrollbar scrollY;
    private Scrollbar scrollX;
    private Scrollbar scrollVitesse;
    
    private List lstStruct;
    
    static int zoom = 2;
    static int offsetX = 0;
    static int offsetY = 0;
    static int vitesse = 20;
    static int nbGen = 0;
    
    static final int TAILLE = 400;
    
    static Label lblZoom = new Label("Zoom = "+zoom);
    static Label lblVitesse = new Label("Speed = "+vitesse);
    static Label lblGeneration = new Label("Gen n"+0);
    
    private static final String LIFSDIR = "lifs/";
    private static final String LIST = "list.txt";
   
    /** Initialisation de l'applet */
    public void init() {
        tab1 = new boolean[TAILLE][TAILLE];
        tab2 = new boolean[TAILLE][TAILLE];
        
        this.setLayout(new BorderLayout());
        
        Panel pnlSouth = new Panel(new BorderLayout());
        Panel pnlNorth = new Panel(new FlowLayout());
        Panel pnlEast = new Panel();
        Panel pnlWest = new Panel(new BorderLayout());
        
        Panel pnlMenu = new Panel(new FlowLayout());
        Panel pnlGeneral = new Panel(new BorderLayout());
        Panel pnlStruct = new Panel(new BorderLayout());
        Panel pnlInfos = new Panel(new GridLayout(4,1));
        
        jeu = new CanvasJeu();
        cmdPlay = new Button("  jouer  ");
        cmdNew = new Button("nouveau");
        cmdCenter = new Button("centrer");
        
        scrollY = new Scrollbar(Scrollbar.VERTICAL, 0, 1, -10, 10);
        scrollX = new Scrollbar(Scrollbar.HORIZONTAL, 0, 1, -10, 10);
        scrollZoom = new Scrollbar(Scrollbar.HORIZONTAL, zoom, 1, 1, 6);
        scrollVitesse = new Scrollbar(Scrollbar.HORIZONTAL, vitesse, 1, 0, 1000);
        scrollVitesse.setUnitIncrement(10);
        
        lstStruct = new List();
        initList();
        
        pnlMenu.add(cmdPlay);
        pnlMenu.add(cmdNew);
        pnlMenu.add(cmdCenter);
        pnlMenu.add(new Label("zoom:"));
        pnlMenu.add(scrollZoom);
        pnlMenu.add(new Label("speed:"));
        pnlMenu.add(scrollVitesse);
        
        pnlSouth.add(pnlMenu,BorderLayout.SOUTH);
        pnlSouth.add(scrollX,BorderLayout.CENTER);
        pnlSouth.add(new Panel(),BorderLayout.EAST);
        pnlSouth.add(new Panel(),BorderLayout.WEST);
        
        Label lblTitre = new Label("JdVie v1.0 (Jeu de la Vie)");
        lblTitre.setFont(new Font("Arial",Font.BOLD,12));
        pnlNorth.add(lblTitre);
        
        pnlWest.add(scrollY,BorderLayout.CENTER);
        pnlWest.add(new Panel(),BorderLayout.WEST);
        
        pnlGeneral.add(pnlSouth,BorderLayout.SOUTH);
        pnlGeneral.add(pnlNorth,BorderLayout.NORTH);
        pnlGeneral.add(pnlEast,BorderLayout.EAST);
        pnlGeneral.add(pnlWest,BorderLayout.WEST);
        pnlGeneral.add(jeu,BorderLayout.CENTER);
        
        pnlInfos.add(lblZoom);
        pnlInfos.add(lblVitesse);
        pnlInfos.add(lblGeneration);
        
        pnlStruct.add(lstStruct,BorderLayout.CENTER);
        pnlStruct.add(new Label("Structures:"),BorderLayout.NORTH);
        pnlStruct.add(new Panel(),BorderLayout.EAST);
        pnlStruct.add(pnlInfos,BorderLayout.SOUTH);
        
        this.add(pnlGeneral,BorderLayout.CENTER);
        this.add(pnlStruct,BorderLayout.EAST);
        
        scrollX.addAdjustmentListener(this);
        scrollY.addAdjustmentListener(this);
        scrollZoom.addAdjustmentListener(this);
        scrollVitesse.addAdjustmentListener(this);
        
        cmdPlay.addActionListener(this);
        cmdNew.addActionListener(this);
        cmdCenter.addActionListener(this);
        
        this.addMouseListener(this);
        jeu.addMouseListener(this);
        
        cmdPlay.setEnabled(true);
        cmdNew.setEnabled(true);
        cmdCenter.setEnabled(true);
        
        initTab(tab1);
        jeu.setTab(tab1);
    }
    
    /** Dmarrage de l'applet */
    public void start() {
        jeu.initVariables(this.getGraphics());
        jeu.update(this.getGraphics());
        gen = null;
    }
    
    /** Arrt de l'applet */
    public void stop() {
        if (gen != null) gen.interrupt();
        cmdPlay.setEnabled(false);
        cmdNew.setEnabled(false);
    }
    
    /** Gestion des boutons */
    public void actionPerformed(java.awt.event.ActionEvent evt) {
        Object cmd = evt.getSource();
        if (cmd.equals(cmdNew)) {
            synchronized(jeu) {
                String s = lstStruct.getSelectedItem();
                if (s != null) {
                    chargeLifFile(tab1,s);
                    copyTab(tab2,tab1);
                    nbGen = 0;
                    lblGeneration.setText("Gen n"+ 0);
                    jeu.repaint();
                    gen = null;
                }
            }
        }
        else if (cmd.equals(cmdPlay)) {
            String action = evt.getActionCommand();
            if (action.equals("  jouer  ")) {
                cmdPlay.setLabel("stopper");
                cmdNew.setEnabled(false);
                if (gen == null) {
                    gen = new Generateur(tab1, tab2, jeu);
                    gen.start();
                }
                else gen.resumeRequest();
            }
            else {
                cmdPlay.setLabel("  jouer  ");
                cmdNew.setEnabled(true);
                gen.suspendRequest();
            }
        }
        else if (cmd.equals(cmdCenter)) {
            offsetX = 0;
            offsetY = 0;
            jeu.repaint();
        }
    }
    
    /** Gestion des barres de dfilement */
    public void adjustmentValueChanged(java.awt.event.AdjustmentEvent evt) {
        Object obj = evt.getSource();
        if (obj.equals(scrollX)) {
            offsetX -= evt.getValue() * (zoom*4);
            scrollX.setValue(0);
            jeu.repaint();
        }
        else if (obj.equals(scrollY)) {
            offsetY -= evt.getValue() * (zoom*4);
            scrollY.setValue(0);
            jeu.repaint();
        }
        else if (obj.equals(scrollZoom)) {
            zoom = evt.getValue();
            lblZoom.setText("Zoom = "+zoom);
            jeu.repaint();
        }
        else if (obj.equals(scrollVitesse)) {
            vitesse = evt.getValue();
            lblVitesse.setText("Speed = "+vitesse);
        }
    }
    
    /** Copie le tableau tab2 dans tab1 */
    private static void copyTab(boolean[][] tab1, boolean[][] tab2) {
        for (int k=0; k<TAILLE; k++)
            for (int j=0; j<TAILLE; j++)
                tab1[k][j] = tab2[k][j];
    }
    
    /** Initialise le tableau */
    private static void initTab(boolean[][] tab) {
        for (int k=0; k<TAILLE; k++)
            for (int j=0; j<TAILLE; j++)
                tab[k][j] = false;
    }
    
    /** Initialise la liste des structures */
    public void initList() {
        String line = null;
        try {
            InputStream fluxEntree = this.getClass().getResourceAsStream(LIFSDIR + LIST);
            BufferedReader fluxDonnees = new BufferedReader(new InputStreamReader(fluxEntree));
            while ((line = fluxDonnees.readLine()) != null)
                lstStruct.add(line.trim().toLowerCase());
        }
        catch(IOException ioe) {
            this.showStatus("Error " + ioe);
        }
    }
    
    /** Charge un fichier lif dans un tableau */
    public void chargeLifFile(boolean [][] tab, String filename) {
        String line = null;
        try {
            initTab(tab);
            int centre = TAILLE/2;
            int x = 0;
            int y = 0;
            StringTokenizer st;
            InputStream fluxEntree = this.getClass().getResourceAsStream(LIFSDIR + filename + ".lif");
            BufferedReader fluxDonnees = new BufferedReader(new InputStreamReader(fluxEntree));
            int j = 0;
            while ((line = fluxDonnees.readLine()) != null) {
                if(!line.equals("")) {
                    if (line.charAt(0) == '#' && line.charAt(1) == 'P') {
                        st = new StringTokenizer(line);
                        st.nextToken();
                        x = Integer.parseInt(st.nextToken());
                        y = Integer.parseInt(st.nextToken());
                        j = 0;
                    }
                    else if(line.charAt(0) == '.' || line.charAt(0) == '*') {
                        
                        for (int i=0 ; i<line.length() ; i++)
                            tab[centre+x+i][centre+y+j] = (line.charAt(i)=='*');
                        j++;
                    }
                }
            }
        }
        catch(IOException ioe) {
            this.showStatus("Error " + ioe);
        }
    }
    
    public void mouseEntered(java.awt.event.MouseEvent evt) {
        this.showStatus("Copyright  2002 Vouriot Thierry, Web: http://thierry.vouriot.free.fr");
    }
    public void mouseExited(java.awt.event.MouseEvent mouseEvent) {
    }
    
    public void mouseReleased(java.awt.event.MouseEvent mouseEvent) {
    }
    
    public void mousePressed(java.awt.event.MouseEvent mouseEvent) {
    }
    
    public void mouseClicked(java.awt.event.MouseEvent mouseEvent) {
    }
}
